@@ -36,7 +36,9 @@
 #include "mlview-preferences.h"
 #include "mlview-drop-manager.h"
 #include "mlview-iview.h"
-
+#include "recent-files/egg-recent.h"
+#include "recent-files/egg-recent-view-uimanager.h"
+#include "mlview-service.h"
 #define ENABLE_NLS 1
 
 /**
@@ -52,6 +54,8 @@
 struct _MlViewAppPriv {
         struct MlViewWidgetsHandle *widgets ;
         GtkUIManager *ui_manager ;
+	EggRecentViewUIManager *recent_view;
+	EggRecentModel *recent_model;
         GtkActionGroup *doc_required_action_group ;
         GtkActionGroup *doc_not_required_action_group ;
         guint main_menubar_merge_id ;
@@ -91,118 +95,133 @@ static enum MlViewStatus init_editor (MlViewApp *a_app,
                                       GladeXML *a_glade_xml) ;
 
 
-static void mlview_app_new_menuitem_action_cb (GtkAction *a_action,
+static void new_menuitem_action_cb (GtkAction *a_action,
                                                MlViewApp *a_user_data) ;
 
-static void mlview_app_open_menuitem_action_cb (GtkAction *a_action,
+static void open_menuitem_action_cb (GtkAction *a_action,
                                                 MlViewApp *a_user_data) ;
 
-static void mlview_app_openlocation_menuitem_action_cb (GtkAction *a_action,
+static void openlocation_menuitem_action_cb (GtkAction *a_action,
                                                         MlViewApp *a_user_data) ;
 
-static void mlview_app_save_menuitem_action_cb (GtkAction *a_action,
+static void save_menuitem_action_cb (GtkAction *a_action,
                                                 MlViewApp *a_user_data) ;
 
-static void mlview_app_save_as_menuitem_action_cb (GtkAction *a_action,
+static void save_as_menuitem_action_cb (GtkAction *a_action,
                                                    MlViewApp *a_user_data) ;
 
-static void mlview_app_close_menuitem_action_cb (GtkAction *a_action,
+static void close_menuitem_action_cb (GtkAction *a_action,
                                                  MlViewApp *a_user_data) ;
 
-static void mlview_app_quit_menuitem_action_cb (GtkAction *a_action,
+static void quit_menuitem_action_cb (GtkAction *a_action,
                                                 MlViewApp *a_user_data) ;
 
-static void mlview_app_undo_menuitem_action_cb (GtkAction *a_action,
+static void undo_menuitem_action_cb (GtkAction *a_action,
                                                 MlViewApp *a_user_data) ;
 
-static void mlview_app_redo_menuitem_action_cb (GtkAction *a_action,
+static void redo_menuitem_action_cb (GtkAction *a_action,
                                                 MlViewApp *a_user_data) ;
 
-static void mlview_app_associated_schemas_menuitem_action_cb (GtkAction *a_action,
+static void associated_schemas_menuitem_action_cb (GtkAction *a_action,
                                                               MlViewApp *a_app) ;
 
-static void mlview_app_new_view_on_doc_menuitem_action_cb (GtkAction *a_action,
+static void new_view_on_doc_menuitem_action_cb (GtkAction *a_action,
                                                            MlViewApp *a_user_data) ;
 
-static void mlview_app_rename_view_menuitem_action_cb (GtkAction *a_action,
+static void rename_view_menuitem_action_cb (GtkAction *a_action,
                                                        MlViewApp *a_user_data) ;
 
 
-static void mlview_app_validate_dtd_menuitem_action_cb (GtkAction *a_action,
+static void validated_dtd_menuitem_action_cb (GtkAction *a_action,
                                                         MlViewApp *a_user_data) ;
 
-static void mlview_app_apply_xslt_dtd_menuitem_action_cb (GtkAction *a_action,
+static void apply_xslt_menuitem_action_cb (GtkAction *a_action,
                                                           MlViewApp *a_user_data) ;
 
-static void mlview_app_about_menuitem_action_cb (GtkAction *a_action,
+static void about_menuitem_action_cb (GtkAction *a_action,
                                                  MlViewApp *a_user_data) ;
 
-static void mlview_app_edit_menu_action_cb (GtkAction *a_action,
+static void edit_menuitem_action_cb (GtkAction *a_action,
                                             MlViewApp *a_user_data) ;
 
+static void reload_document_menuitem_action_cb (GtkAction *a_action,
+							   MlViewApp *a_app) ;
+
+static void open_recent_cb (GtkAction *a_action,
+					MlViewApp *a_user_data);
+
+static enum MlViewStatus build_view_types_choice_submenu (MlViewApp *a_this) ;
+
 /*actions that require a document to be open*/
 static GtkActionEntry gv_doc_required_actions [] = {
 
         /*File Menu*/
         {
-                "SaveMenuitemAction", GTK_STOCK_SAVE, N_("_Save"), 
+                "SaveMenuitemAction", GTK_STOCK_SAVE, NULL, 
                 "<control>S", N_("Save the XML Document"), 
-                G_CALLBACK (mlview_app_save_menuitem_action_cb)},
-
+                G_CALLBACK (save_menuitem_action_cb)
+	},
         {
-                "SaveAsMenuitemAction", GTK_STOCK_SAVE_AS, N_("Save _As"), 
+                "SaveAsMenuitemAction", GTK_STOCK_SAVE_AS, NULL, 
                 "<shift><control>S", N_("Save the XML Document in an other file"), 
-                G_CALLBACK (mlview_app_save_as_menuitem_action_cb)},
+                G_CALLBACK (save_as_menuitem_action_cb)
+	},
 
         {
-                "CloseMenuitemAction", GTK_STOCK_CLOSE, N_("_Close"), 
+                "ReloadDocumentMenuitemAction", GTK_STOCK_REVERT_TO_SAVED, NULL, 
+                "<control>R", N_("Reload the XML Document saved on disk"), 
+                G_CALLBACK (reload_document_menuitem_action_cb)
+	},
+        {
+                "CloseMenuitemAction", GTK_STOCK_CLOSE, NULL, 
                 "<control>W", N_("Close the XML Document"), 
-                G_CALLBACK (mlview_app_close_menuitem_action_cb)},
+                G_CALLBACK (close_menuitem_action_cb)
+	},
         
         /*Edit Menu*/        
         {
                 "EditMenuAction", NULL, N_("_Edit"), NULL, NULL, 
-                G_CALLBACK (mlview_app_edit_menu_action_cb)
+                G_CALLBACK (edit_menuitem_action_cb)
         },
 
         {
-                "UndoMenuitemAction", GTK_STOCK_UNDO, N_("_Undo"), 
+                "UndoMenuitemAction", GTK_STOCK_UNDO, NULL, 
                 "<control>Z", N_("Undo the last action"), 
-                G_CALLBACK (mlview_app_undo_menuitem_action_cb)},
+                G_CALLBACK (undo_menuitem_action_cb)},
         
         {
-                "RedoMenuitemAction", GTK_STOCK_REDO, N_("_Redo"), 
+                "RedoMenuitemAction", GTK_STOCK_REDO, NULL, 
                 "<shift><control>Z", N_("Undo the last action"),
-                G_CALLBACK (mlview_app_redo_menuitem_action_cb)},
-        /*
-          {
-                "FindMenuitemAction", GTK_STOCK_FIND, N_("Find node"),
-                "<control>F", N_("Finds a node in the document"),
-                G_CALLBACK (mlview_app_find_node_menuitem_action_cb) ;
-                }*/
+                G_CALLBACK (redo_menuitem_action_cb)
+	},
 
         /*Tools menu*/
         {"ToolsMenuAction", NULL, N_("_Tools"), NULL, NULL, NULL},
 
         {"NewViewOnDocMenuitemAction", NULL, N_("New _view"), 
          NULL, N_("Creates a new view on the current document"), 
-         G_CALLBACK (mlview_app_new_view_on_doc_menuitem_action_cb)},
-        
+         NULL, /*G_CALLBACK (new_view_on_doc_menuitem_action_cb)*/
+	},
+
         {"RenameViewMenuitemAction", NULL, N_("Re_name view"), 
          NULL, N_("Rename the view"), 
-         G_CALLBACK (mlview_app_rename_view_menuitem_action_cb)},
+         G_CALLBACK (rename_view_menuitem_action_cb)
+	},
 
-        {"AssociatedSchemasMenuitemAction", NULL, N_("Schemas"), 
+        {"AssociatedSchemasMenuitemAction", GTK_STOCK_DND, N_("Schemas"), 
          NULL, N_("Manage the schemas/DTDs associated to the document"), 
-         G_CALLBACK (mlview_app_associated_schemas_menuitem_action_cb)},
+         G_CALLBACK (associated_schemas_menuitem_action_cb)
+	},
 
-        {"ValidateDocMenuitemAction", NULL, N_("Va_lidate document"),
+        {"ValidateDocMenuitemAction", GTK_STOCK_APPLY, N_("Va_lidate document"),
          NULL, N_("Validate the document against the associated DTD"), 
-         G_CALLBACK (mlview_app_validate_dtd_menuitem_action_cb)},
+         G_CALLBACK (validated_dtd_menuitem_action_cb)
+	},
 
-        {"ApplyXSLTMenuitemAction", NULL, N_("Apply an _XSL transform"),
+        {"ApplyXSLTMenuitemAction", GTK_STOCK_CONVERT, N_("Apply an _XSL transform"),
          NULL, N_("Apply an XSL transform to the current document"), 
-         G_CALLBACK (mlview_app_apply_xslt_dtd_menuitem_action_cb)},
+         G_CALLBACK (apply_xslt_menuitem_action_cb)
+	},
 } ;
 
 
@@ -210,30 +229,86 @@ static GtkActionEntry gv_doc_not_required_actions[] = {
         /*File Menu*/
         {"FileMenuAction", NULL, N_("_File"), NULL, NULL, NULL},
 
-        {"NewMenuitemAction", GTK_STOCK_NEW, N_("_New"), 
+        {"NewMenuitemAction", GTK_STOCK_NEW, NULL, 
          "<control>N", N_("Create a new XML Document"), 
-         G_CALLBACK (mlview_app_new_menuitem_action_cb)},
+         G_CALLBACK (new_menuitem_action_cb)},
 
-        {"OpenMenuitemAction", GTK_STOCK_OPEN, N_("_Open"), 
+        {"OpenMenuitemAction", GTK_STOCK_OPEN, NULL, 
          "<control>O", N_("Open an XML Document"), 
-         G_CALLBACK (mlview_app_open_menuitem_action_cb)},
+         G_CALLBACK (open_menuitem_action_cb)},
 
-        {"OpenLocationMenuitemAction", 0, N_("Open _location"),
+        {"OpenLocationMenuitemAction", GTK_STOCK_NETWORK, N_("Open _location"),
          "<control>L", N_("Open an XML Document from an URI"), 
-         G_CALLBACK (mlview_app_openlocation_menuitem_action_cb)},
+         G_CALLBACK (openlocation_menuitem_action_cb)},
         
-        {"QuitMenuitemAction", GTK_STOCK_QUIT, N_("_Quit"), 
+        {"QuitMenuitemAction", GTK_STOCK_QUIT, NULL, 
          "<control>Q", N_("Quit the MlView application"), 
-         G_CALLBACK (mlview_app_quit_menuitem_action_cb)},
+         G_CALLBACK (quit_menuitem_action_cb)},
                 
         /*Help Menu*/
         {"HelpMenuAction", NULL, N_("_Help"), NULL, NULL, NULL},
 
-        {"AboutMenuitemAction", GNOME_STOCK_ABOUT, N_("_About"),
+        {"AboutMenuitemAction", GNOME_STOCK_ABOUT, NULL,
          NULL, N_("About MlView ..."), 
-         G_CALLBACK (mlview_app_about_menuitem_action_cb)}
+         G_CALLBACK (about_menuitem_action_cb)}
 } ;
 
+/**
+ * Build the submenu that lets the user choose the types of views
+ * she wants to use.
+ * @param a_this the current instance of #MlViewApp
+ */
+static enum MlViewStatus 
+build_view_types_choice_submenu (MlViewApp *a_this) 
+{
+	gint number_of_views = 0, i=0, merge_id= 0 ;
+	MlViewEditor *editor = NULL ;
+	struct MlViewViewDesc *view_desc = NULL ;
+	GtkUIManager *ui_manager = NULL ;
+
+	g_return_val_if_fail (a_this && PRIVATE (a_this),
+			      MLVIEW_ERROR) ;
+	g_return_val_if_fail (PRIVATE (a_this)->doc_required_action_group,
+			      MLVIEW_BAD_PARAM_ERROR) ;
+
+	editor = mlview_app_get_editor (a_this) ;
+	g_return_val_if_fail (editor, MLVIEW_ERROR) ;
+	number_of_views = mlview_editor_get_number_of_view_desc () ;
+	if (!number_of_views) {
+		mlview_utils_trace_debug ("Got 0 views") ;
+		return MLVIEW_ERROR ;
+	}
+	ui_manager = mlview_app_get_ui_manager (a_this) ;
+	g_return_val_if_fail (ui_manager, MLVIEW_ERROR) ;
+
+	merge_id = gtk_ui_manager_new_merge_id (ui_manager) ; 
+	for (i=0 ; i < number_of_views ;i++) {
+		GtkAction *action = NULL ;
+		view_desc = mlview_editor_get_view_descriptor_at (i) ; 
+		if (!view_desc) {
+			break ;
+		}
+		action = gtk_action_new (view_desc->view_type_name,
+				         view_desc->translated_view_name,
+					 view_desc->view_description,
+					 NULL) ;
+		gtk_action_group_add_action 
+			(PRIVATE (a_this)->doc_required_action_group,
+			 action) ;
+		g_object_set_data (G_OBJECT (action), "view-desc", view_desc) ;
+		g_signal_connect (G_OBJECT (action),
+				  "activate",
+				  G_CALLBACK (new_view_on_doc_menuitem_action_cb),
+				  a_this) ;
+		gtk_ui_manager_add_ui 
+			(ui_manager, merge_id, 
+		         "/MainMenubar/ToolsMenu/NewViewOnDocMenuitem",
+			 view_desc->view_type_name, view_desc->view_type_name,
+			 GTK_UI_MANAGER_AUTO, FALSE) ;
+		gtk_ui_manager_ensure_update (ui_manager) ;
+	}
+	return MLVIEW_OK ;
+}
 
 /*
  *Displays the about dialog.
@@ -241,7 +316,7 @@ static GtkActionEntry gv_doc_not_required_actions[] = {
 static void
 display_about_dialog ()
 {
-        static GtkWidget *about_dialog = NULL;
+        static GtkWidget *about_dialog = NULL, **widget_ptr = NULL;
         guchar *path = NULL ;
         GdkPixbuf *pixbuf = NULL;
 
@@ -297,8 +372,10 @@ display_about_dialog ()
                 g_object_unref (pixbuf) ;
                 pixbuf = NULL ;
         }
+
+	widget_ptr = &about_dialog ;
         g_object_add_weak_pointer (G_OBJECT (about_dialog),
-                                   (void **) &about_dialog);
+                                   (void **) widget_ptr);
         gtk_widget_show (about_dialog);
 }
 
@@ -306,33 +383,6 @@ display_about_dialog ()
  *General signal handlers for the differents menuitems
  *========================================================*/
 
-static void
-close_application (MlViewApp *a_this)
-{
-        MlViewEditor *editor = NULL;
-        struct MlViewWidgetsHandle *handle = NULL ;
-        gboolean is_ok = FALSE;
-
-        g_return_if_fail (a_this && PRIVATE (a_this));
-        
-        editor = mlview_app_get_editor (a_this);
-        g_return_if_fail (editor) ;
-        
-        handle = mlview_app_get_widgets_handle (a_this) ;
-        g_return_if_fail (handle && handle->app_win) ;
-
-        if (editor)
-                is_ok = mlview_editor_close_all_xml_documents_interactive
-                        (editor);
-
-        if (is_ok == FALSE) {
-                gtk_widget_show (handle->app_win) ;
-                return;
-        }
-
-        gtk_widget_destroy (handle->app_win);
-        gtk_main_quit ();
-}
 
 static gboolean
 delete_event_cb (GtkWidget *a_widget,
@@ -343,7 +393,7 @@ delete_event_cb (GtkWidget *a_widget,
                               && a_app,
                               FALSE) ;
 
-        close_application (a_app) ;
+        mlview_app_close_application (a_app, TRUE) ;
         return TRUE ;
 }
 
@@ -361,9 +411,9 @@ document_name_changed_cb (MlViewAppContext *a_ctxt,
                           && MLVIEW_IS_APP_CONTEXT (a_ctxt)) ;
         g_return_if_fail (a_doc && MLVIEW_IS_XML_DOCUMENT (a_doc)) ;
 
-        doc = mlview_xml_document_get_xml_document (a_doc) ;
+        doc = mlview_xml_document_get_native_document (a_doc) ;
         if (!doc) {
-                mlview_utils_trace_info ("mlview_xml_document_get_xml_document() failed\n") ;
+                mlview_utils_trace_debug ("mlview_xml_document_get_xml_document() failed\n") ;
                 return ;
         }
         if (doc->name) {
@@ -390,9 +440,9 @@ view_swapped_cb (MlViewAppContext *a_ctxt,
 
         mlview_iview_get_document (a_new_view, &mlview_doc) ;
         if (mlview_doc)
-                doc = mlview_xml_document_get_xml_document (mlview_doc) ;
+                doc = mlview_xml_document_get_native_document (mlview_doc) ;
         if (!doc) {
-                mlview_utils_trace_info 
+                mlview_utils_trace_debug 
                         ("mlview_xml_document_get_xml_document() failed\n") ;
                 return ;
         }
@@ -405,9 +455,8 @@ view_swapped_cb (MlViewAppContext *a_ctxt,
 }
 
 static void
-document_undo_state_changed_cb (MlViewAppContext *a_ctxt,
-                                MlViewXMLDocument *a_doc,
-                                MlViewApp *a_app)
+view_undo_state_changed_cb (MlViewAppContext *a_ctxt,
+			    MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
         GtkUIManager *ui_manager = NULL ;
@@ -420,33 +469,27 @@ document_undo_state_changed_cb (MlViewAppContext *a_ctxt,
 
         editor = mlview_app_context_get_element (a_ctxt, "MlViewEditor") ;
         if (!editor) {
-                mlview_utils_trace_info ("Could not find the editor") ; 
+                mlview_utils_trace_debug ("Could not find the editor") ; 
                 return ;
         }
 
         ui_manager = mlview_app_get_ui_manager (a_app) ;
         g_return_if_fail (ui_manager) ;
 
-        undo_action = gtk_ui_manager_get_action (ui_manager, "/MainToolbar/UndoToolitem") ;
+        undo_action = gtk_ui_manager_get_action (ui_manager, 
+						 "/MainToolbar/UndoToolitem") ;
         g_return_if_fail (undo_action) ;
-        redo_action = gtk_ui_manager_get_action (ui_manager, "/MainToolbar/RedoToolitem") ;
+        redo_action = gtk_ui_manager_get_action (ui_manager, 
+						 "/MainToolbar/RedoToolitem") ;
 
-        if (!a_doc || !MLVIEW_IS_XML_DOCUMENT (a_doc)) {
-                g_object_set (G_OBJECT (undo_action),
-                              "sensitive", FALSE, NULL) ;
-                g_object_set (G_OBJECT (redo_action),
-                              "sensitive", FALSE, NULL) ;
-                return ;
-        }
-
-        if (mlview_xml_document_can_undo_mutation (a_doc) == TRUE) {
+        if (mlview_editor_can_undo (editor) == TRUE) {
                 g_object_set (G_OBJECT (undo_action),
                               "sensitive", TRUE, NULL) ;
         } else {
                 g_object_set (G_OBJECT (undo_action),
                               "sensitive", FALSE, NULL) ;
         }
-        if (mlview_xml_document_can_redo_mutation (a_doc) == TRUE) {
+        if (mlview_editor_can_redo (editor) == TRUE) {
                 g_object_set (G_OBJECT (redo_action), "sensitive", TRUE, NULL) ;
                 
         } else {
@@ -460,7 +503,6 @@ set_editing_enabled (MlViewApp *a_this,
 {
         GtkUIManager *ui_manager = NULL ;
         MlViewEditor *editor = NULL ;
-        MlViewXMLDocument *doc = NULL ;
         MlViewAppContext *context = NULL ;
 
         g_return_if_fail (a_this && PRIVATE (a_this)) ;
@@ -478,15 +520,15 @@ set_editing_enabled (MlViewApp *a_this,
                 gtk_action_group_set_sensitive (PRIVATE (a_this)->doc_required_action_group,
                                                 FALSE) ;
         }
+
         /*now update the undo/redo action sensitivity*/
         editor = mlview_app_get_editor (a_this) ;
         if (!editor)
                 return ;
 
-        doc = mlview_editor_get_cur_doc (editor) ;
         context = mlview_editor_get_app_context (editor) ;
         g_return_if_fail (context) ;
-        mlview_app_context_notify_document_undo_state_changed (context, doc) ;
+        mlview_app_context_notify_view_undo_state_changed (context) ;
 }
 
 static void
@@ -529,7 +571,7 @@ first_view_added_cb (MlViewEditor *a_editor,
 
 
 static void 
-mlview_app_new_menuitem_action_cb (GtkAction *a_action,
+new_menuitem_action_cb (GtkAction *a_action,
                                    MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -542,7 +584,7 @@ mlview_app_new_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_open_menuitem_action_cb (GtkAction *a_action,
+open_menuitem_action_cb (GtkAction *a_action,
                                     MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -555,7 +597,7 @@ mlview_app_open_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void
-mlview_app_openlocation_menuitem_action_cb (GtkAction *a_action,
+openlocation_menuitem_action_cb (GtkAction *a_action,
                                             MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -567,8 +609,39 @@ mlview_app_openlocation_menuitem_action_cb (GtkAction *a_action,
         mlview_editor_open_xml_document_interactive (editor) ;
 }
 
+static void
+open_recent_cb (
+		GtkAction *a_action,
+		MlViewApp *a_app)
+{
+	MlViewEditor *editor = NULL;
+	const EggRecentItem *item;
+	gchar *uri;
+
+	g_return_if_fail (a_action && a_app && PRIVATE (a_app));
+	
+	item = egg_recent_view_uimanager_get_item (PRIVATE (a_app)->recent_view, a_action);
+	uri = (gchar*)egg_recent_item_peek_uri (item);
+
+	editor = mlview_app_get_editor (a_app);
+	g_return_if_fail (editor && MLVIEW_IS_EDITOR (editor)) ;
+
+	mlview_editor_load_xml_file (editor, uri, TRUE);
+}
+
+static char*
+tooltip_cb (EggRecentItem *item, gpointer user_data)
+{
+        char *uri = NULL;
+
+        uri = egg_recent_item_get_uri_for_display (item);
+
+        return uri;
+}
+
+
 static void 
-mlview_app_save_menuitem_action_cb (GtkAction *a_action,
+save_menuitem_action_cb (GtkAction *a_action,
                                     MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -581,7 +654,7 @@ mlview_app_save_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_save_as_menuitem_action_cb (GtkAction *a_action,
+save_as_menuitem_action_cb (GtkAction *a_action,
                                        MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -594,7 +667,7 @@ mlview_app_save_as_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_close_menuitem_action_cb (GtkAction *a_action,
+close_menuitem_action_cb (GtkAction *a_action,
                                      MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -603,11 +676,11 @@ mlview_app_close_menuitem_action_cb (GtkAction *a_action,
         editor = mlview_app_get_editor (a_app) ;
         g_return_if_fail (editor && MLVIEW_IS_EDITOR (editor)) ;
 
-        mlview_editor_close_xml_document_interactive (editor) ;
+        mlview_editor_close_xml_document (editor, TRUE) ;
 }
 
 static void 
-mlview_app_quit_menuitem_action_cb (GtkAction *a_action,
+quit_menuitem_action_cb (GtkAction *a_action,
                                     MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -620,11 +693,11 @@ mlview_app_quit_menuitem_action_cb (GtkAction *a_action,
         editor = mlview_app_get_editor (a_app) ;
         g_return_if_fail (editor && MLVIEW_IS_EDITOR (editor)) ;
 
-        close_application (a_app) ;
+        mlview_app_close_application (a_app, TRUE) ;
 }
 
 static void 
-mlview_app_undo_menuitem_action_cb (GtkAction *a_action,
+undo_menuitem_action_cb (GtkAction *a_action,
                                     MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -637,7 +710,7 @@ mlview_app_undo_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_redo_menuitem_action_cb (GtkAction *a_action,
+redo_menuitem_action_cb (GtkAction *a_action,
                                     MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -650,17 +723,28 @@ mlview_app_redo_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_new_view_on_doc_menuitem_action_cb (GtkAction *a_action,
-                                               MlViewApp *a_app)
+new_view_on_doc_menuitem_action_cb (GtkAction *a_action,
+                                    MlViewApp *a_app)
 {
+	struct MlViewViewDesc *view_desc = NULL ; 
         MlViewEditor *editor = NULL ;
+	MlViewIView *view = NULL;  
+
         g_return_if_fail (a_action && a_app && PRIVATE (a_app)) ;
         
         editor = mlview_app_get_editor (a_app) ;
         g_return_if_fail (editor && MLVIEW_IS_EDITOR (editor)) ;
-
-        mlview_editor_create_new_view_on_current_document_interactive
-                (editor);
+	view_desc = g_object_get_data (G_OBJECT (a_action), "view-desc") ;
+	if (!view_desc) {
+		mlview_utils_trace_debug ("Could not get the correct view desc") ;
+		return ;
+	}
+	view = mlview_editor_create_new_view_on_current_document (editor, view_desc) ;
+	if (!view) {
+		mlview_utils_trace_debug ("Could not create the view requested") ;
+		return ;
+	}
+	mlview_editor_add_view (editor, view) ;
 }
 
 /*
@@ -682,7 +766,7 @@ mlview_app_find_node_menuitem_action_cb (GtkAction *a_action,
 */
 
 static void 
-mlview_app_rename_view_menuitem_action_cb (GtkAction *a_action,
+rename_view_menuitem_action_cb (GtkAction *a_action,
                                            MlViewApp *a_app) 
 {
         MlViewEditor *editor = NULL ;
@@ -695,7 +779,7 @@ mlview_app_rename_view_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void
-mlview_app_associated_schemas_menuitem_action_cb (GtkAction *a_action,
+associated_schemas_menuitem_action_cb (GtkAction *a_action,
                                                   MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -708,7 +792,7 @@ mlview_app_associated_schemas_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_validate_dtd_menuitem_action_cb (GtkAction *a_action,
+validated_dtd_menuitem_action_cb (GtkAction *a_action,
                                             MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -721,7 +805,7 @@ mlview_app_validate_dtd_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_apply_xslt_dtd_menuitem_action_cb (GtkAction *a_action,
+apply_xslt_menuitem_action_cb (GtkAction *a_action,
                                               MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -735,7 +819,7 @@ mlview_app_apply_xslt_dtd_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_about_menuitem_action_cb (GtkAction *a_action,
+about_menuitem_action_cb (GtkAction *a_action,
                                      MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -749,7 +833,7 @@ mlview_app_about_menuitem_action_cb (GtkAction *a_action,
 }
 
 static void 
-mlview_app_edit_menu_action_cb (GtkAction *a_action,
+edit_menuitem_action_cb (GtkAction *a_action,
                                 MlViewApp *a_app)
 {
         MlViewEditor *editor = NULL ;
@@ -763,6 +847,21 @@ mlview_app_edit_menu_action_cb (GtkAction *a_action,
         mlview_editor_make_current_view_populate_application_edit_menu (editor) ;
 }
 
+static void
+reload_document_menuitem_action_cb (GtkAction *a_action,
+		                               MlViewApp *a_app)
+{
+	MlViewEditor *editor = NULL ;
+
+	g_return_if_fail (a_action && GTK_IS_ACTION (a_action)
+			  && a_app && PRIVATE (a_app)) ;
+
+	editor = mlview_app_get_editor (a_app) ;
+	g_return_if_fail (editor && MLVIEW_IS_EDITOR (editor)) ;
+
+         mlview_editor_reload_document (editor, TRUE) ;  
+}
+
 /**
  *Make sure all the signals of the menu items
  *the right function callbacks are correctly
@@ -805,8 +904,9 @@ build_and_init_menus (MlViewApp *a_this,
         action_group = gtk_action_group_new ("DocRequiredActions") ;
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE) ;
 
-        gtk_action_group_add_actions (action_group, gv_doc_required_actions,
-                                      sizeof (gv_doc_required_actions)/sizeof (GtkActionEntry),
+        gtk_action_group_add_actions 
+			(action_group, gv_doc_required_actions,
+		         sizeof (gv_doc_required_actions)/sizeof (GtkActionEntry),
                                       a_this) ;
         PRIVATE (a_this)->doc_required_action_group = action_group ;
         
@@ -852,6 +952,11 @@ build_and_init_menus (MlViewApp *a_this,
                 file_path = NULL ;
         }
 
+	/*
+	 * call the dynamic view types choice submenu building code
+	 */
+	build_view_types_choice_submenu (a_this) ;
+
         menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenubar") ;
         g_return_val_if_fail (menubar, MLVIEW_ERROR) ;
         gtk_widget_show_all (menubar) ;
@@ -876,8 +981,11 @@ init_app_win (MlViewApp *a_app,
               GladeXML *a_glade_xml)
 {
         MlViewAppContext *ctxt = NULL ;
+	struct MlViewAppSettings *settings = NULL ;
         GtkWidget *editor_container = NULL ;
         gchar *mlview_icon_filename = NULL ;
+	EggRecentModel *model = NULL;
+	EggRecentViewUIManager *view = NULL;
         enum MlViewStatus status = MLVIEW_ERROR ;
 
         g_return_val_if_fail (a_glade_xml && a_app,
@@ -904,18 +1012,37 @@ init_app_win (MlViewApp *a_app,
         g_return_val_if_fail (PRIVATE (a_app)->widgets->editor,
                               MLVIEW_ERROR) ;
 
+	/*
+	 * Add recent file entries
+	 */
+	model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
+	egg_recent_model_set_filter_mime_types (model, "text/xml", "text/html", NULL);
+
+	view = egg_recent_view_uimanager_new (
+			PRIVATE (a_app)->ui_manager,
+			"/MainMenubar/FileMenu/OpenRecent",
+			NULL,
+			NULL);
+	egg_recent_view_uimanager_set_action_func (
+			view,
+			G_CALLBACK (open_recent_cb),
+			a_app);
+		
+	egg_recent_view_uimanager_set_tooltip_func (view, tooltip_cb, NULL);
+        egg_recent_view_uimanager_show_icons (view, FALSE);
+	egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+	PRIVATE (a_app)->recent_model = model;
+	PRIVATE (a_app)->recent_view = view;
+
         ctxt = mlview_editor_get_app_context 
                 (MLVIEW_EDITOR (PRIVATE (a_app)->widgets->editor)) ;
         g_return_val_if_fail (ctxt, MLVIEW_ERROR) ;
 
-        g_object_set (G_OBJECT (PRIVATE (a_app)->widgets->app_win),
-                      "allow-shrink", TRUE, NULL) ;
-        
         /*
          * Add Drop Support to window
          */
         mlview_drop_manager_register_target (ctxt, GTK_WIDGET (PRIVATE (a_app)->widgets->app_win));
-        
+
         /*
          *set the main window  and other mlview component 
 	 *to the context. That way, components willing
@@ -937,19 +1064,45 @@ init_app_win (MlViewApp *a_app,
                                         a_app) ;
         mlview_app_context_set_element (ctxt, "MlViewUIManager",
                                         PRIVATE (a_app)->ui_manager) ;
+	mlview_app_context_set_element (
+			ctxt, 
+			"MlViewRecentModel",
+			PRIVATE (a_app)->recent_model);
+			
 
+        mlview_icon_filename = g_build_filename (DATADIR, 
+			                         "pixmaps", 
+						 "mlview-app-icon.png",
+			                          NULL) ; 
+	g_return_val_if_fail (mlview_icon_filename, MLVIEW_ERROR) ;
+
+	if (g_file_test (mlview_icon_filename, G_FILE_TEST_EXISTS) != TRUE) {
+		g_warning ("could not find file %s\n", 
+			   mlview_icon_filename) ;
+		return MLVIEW_ERROR ;
+	}
 
-        mlview_icon_filename = gnome_program_locate_file
-                (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
-                 PACKAGE "/mlview-app-icon.png", TRUE,
-                 NULL) ;
-        g_return_val_if_fail (mlview_icon_filename, MLVIEW_ERROR) ;
         gtk_window_set_icon_from_file 
                 (GTK_WINDOW (PRIVATE (a_app)->widgets->app_win),
                  mlview_icon_filename, NULL) ;
         gtk_window_set_default_icon_from_file 
                 (mlview_icon_filename, NULL) ;
 
+	if (mlview_icon_filename) {
+		g_free (mlview_icon_filename) ;
+		mlview_icon_filename = NULL ;
+	}
+
+	/*
+	 * Restore window size
+	 */
+	settings = mlview_app_context_get_settings (ctxt);
+		
+	gtk_window_resize (
+		GTK_WINDOW (PRIVATE (a_app)->widgets->app_win),
+		settings->sizes.main_window_width,
+		settings->sizes.main_window_height);
+
 	/**
 	 * Connect to usefull signal to give "life" to the
 	 * application
@@ -985,8 +1138,8 @@ init_app_win (MlViewApp *a_app,
                           a_app) ;
 
         g_signal_connect (G_OBJECT (ctxt),
-                          "document-undo-state-changed",
-                          G_CALLBACK (document_undo_state_changed_cb),
+                          "view-undo-state-changed",
+                          G_CALLBACK (view_undo_state_changed_cb),
                           a_app) ;
 
         editor_container = NULL ;
@@ -1175,21 +1328,21 @@ mlview_app_new (const guchar *a_appname)
 
         mlview_app = g_try_malloc (sizeof (MlViewApp)) ;
         if (!mlview_app) {
-                mlview_utils_trace_info ("Out of memory") ;
+                mlview_utils_trace_debug ("Out of memory") ;
                 return NULL ;
         }
         memset (mlview_app, 0, sizeof (MlViewApp)) ;        
         
         PRIVATE (mlview_app) = g_try_malloc (sizeof (MlViewAppPriv)) ;
         if (!PRIVATE (mlview_app)) {
-                mlview_utils_trace_info ("Out of memory") ;
+                mlview_utils_trace_debug ("Out of memory") ;
                 goto end ;
         }
         memset (PRIVATE (mlview_app), 0, sizeof (MlViewAppPriv)) ;
         PRIVATE (mlview_app)->widgets = 
                 g_try_malloc (sizeof (struct MlViewWidgetsHandle)) ;
         if (!PRIVATE (mlview_app)->widgets) {
-                mlview_utils_trace_info ("Out of memory") ;
+                mlview_utils_trace_debug ("Out of memory") ;
                 goto end ;
         }
         memset (PRIVATE (mlview_app)->widgets, 0, 
@@ -1251,6 +1404,19 @@ mlview_app_get_application_context (MlViewApp *a_app)
         return ctxt ;
 }
 
+/**
+ * Make the main window visible
+ *
+ * @param a_app the current mlview application
+ */
+void
+mlview_app_set_visible (MlViewApp *a_this)
+{
+	g_return_if_fail (a_this && PRIVATE (a_this));
+
+	gtk_widget_show_all (GTK_WIDGET (PRIVATE (a_this)->widgets->app_win));
+}
+
 /**
  *Getter of the instance of MlViewEditor associated
  *to the current instance of GnomeApp.
@@ -1326,3 +1492,79 @@ mlview_app_get_ui_manager (MlViewApp *a_this)
 
         return PRIVATE (a_this)->ui_manager ;
 }
+
+void
+mlview_app_close_application (MlViewApp *a_this,
+		              gboolean a_interactive)
+{
+        MlViewEditor *editor = NULL;
+        struct MlViewWidgetsHandle *handle = NULL ;
+        gboolean is_ok = FALSE;
+	gint w, h;
+
+        g_return_if_fail (a_this && PRIVATE (a_this));
+        
+        editor = mlview_app_get_editor (a_this);
+        g_return_if_fail (editor) ;
+        
+        handle = mlview_app_get_widgets_handle (a_this) ;
+        g_return_if_fail (handle && handle->app_win) ;
+
+        if (editor)
+                is_ok = mlview_editor_close_all_xml_documents
+                        (editor, a_interactive);
+
+        if (is_ok == FALSE) {
+                gtk_widget_show (handle->app_win) ;
+                return;
+        }
+
+	/*
+	 * Save window size
+	 */
+	gtk_window_get_size (GTK_WINDOW (handle->app_win), &w, &h);
+	mlview_app_context_save_window_state (
+			mlview_app_get_application_context (a_this),
+			w,
+			h);
+	/*mlview_service_stop (a_this, NULL) ; does not work very well*/
+	
+        gtk_widget_destroy (handle->app_win);
+        gtk_main_quit ();
+}
+
+void
+mlview_app_close_all_docs (MlViewApp *a_this,
+		           gboolean a_interactive)
+{
+        MlViewEditor *editor = NULL;
+        struct MlViewWidgetsHandle *handle = NULL ;
+        gboolean is_ok = FALSE;
+	gint w, h;
+
+        g_return_if_fail (a_this && PRIVATE (a_this));
+        
+        editor = mlview_app_get_editor (a_this);
+        g_return_if_fail (editor) ;
+        
+        handle = mlview_app_get_widgets_handle (a_this) ;
+        g_return_if_fail (handle && handle->app_win) ;
+
+        if (editor)
+                is_ok = mlview_editor_close_all_xml_documents
+                        (editor, a_interactive);
+
+        if (is_ok == FALSE) {
+                gtk_widget_show (handle->app_win) ;
+                return;
+        }
+
+	/*
+	 * Save window size
+	 */
+	gtk_window_get_size (GTK_WINDOW (handle->app_win), &w, &h);
+	mlview_app_context_save_window_state (
+			mlview_app_get_application_context (a_this),
+			w,
+			h);
+}
