@@ -38,8 +38,8 @@
  *and toolbars and creates an instance of MlViewAppContext.
  */
 
-#define MLVIEW_ROOTWINDOW_WIDTH 400
-#define MLVIEW_ROOTWINDOW_HEIGHT 300
+#define MLVIEW_ROOTWINDOW_WIDTH 800
+#define MLVIEW_ROOTWINDOW_HEIGHT 600
 #define MLVIEW_MAINPANED_POSITION 400
 
 G_BEGIN_DECLS
@@ -73,6 +73,8 @@ struct MlViewWidgetsHandle {
 
 MlViewApp * mlview_app_new (const guchar * a_appname);
 
+void mlview_app_set_visible (MlViewApp *a_app);
+
 MlViewEditor *mlview_app_get_editor (MlViewApp * a_win);
 
 MlViewAppContext * mlview_app_get_application_context (MlViewApp *a_app) ;
@@ -84,5 +86,10 @@ enum MlViewStatus mlview_app_set_main_window_title (MlViewApp *a_this,
 
 GtkUIManager * mlview_app_get_ui_manager (MlViewApp *a_this) ;
 
+void mlview_app_close_application (MlViewApp *a_this,
+		                   gboolean a_interactive) ;
+
+void mlview_app_close_all_docs (MlViewApp *a_this, gboolean a_interactive) ;
+
 G_END_DECLS
 #endif
