@@ -166,7 +166,7 @@ mlview_kb_eng_alloc_keyinputs_space (MlViewKBEng *a_this)
         
         PRIVATE (a_this)->keyinputs = g_try_malloc (size) ;
         if (!PRIVATE (a_this)->keyinputs) {
-                mlview_utils_trace_info ("system may be out of memory") ;
+                mlview_utils_trace_debug ("system may be out of memory") ;
                 return MLVIEW_OUT_OF_MEMORY_ERROR ;
         }
         memset (PRIVATE (a_this)->keyinputs, 0, size) ;
@@ -194,7 +194,7 @@ mlview_kb_eng_grow_keyinputs_space (MlViewKBEng *a_this)
         
         new_keyinputs = g_try_malloc (size) ;
         if (!new_keyinputs) {
-                mlview_utils_trace_info ("System may be out of memory") ;
+                mlview_utils_trace_debug ("System may be out of memory") ;
                 return MLVIEW_OUT_OF_MEMORY_ERROR ;
         }
         memset (new_keyinputs, 0, size) ;
@@ -217,14 +217,14 @@ mlview_kb_eng_alloc_keybindings_space (MlViewKBEng *a_this)
 			      MLVIEW_BAD_PARAM_ERROR) ;
 
 	if (PRIVATE (a_this)->keybindings) {
-		mlview_utils_trace_info ("PRIVATE (a_this)->keybindings already allocated") ;
+		mlview_utils_trace_debug ("PRIVATE (a_this)->keybindings already allocated") ;
 		return MLVIEW_ERROR ;
 	}
 	PRIVATE (a_this)->keybindings = g_try_malloc 
 		(gv_keybindings_alloc_chunck_size
 		 * sizeof (struct MlViewKBDef)) ;
 	if (!PRIVATE (a_this)->keybindings) {
-		mlview_utils_trace_info ("System may be out of memory") ;
+		mlview_utils_trace_debug ("System may be out of memory") ;
 		return MLVIEW_OUT_OF_MEMORY_ERROR ;
 	}
 	memset (PRIVATE (a_this)->keybindings, 0,
@@ -259,7 +259,7 @@ mlview_kb_eng_grow_keybindings_space (MlViewKBEng *a_this)
 
 	new_keybindings = g_try_malloc (size) ;
 	if (!new_keybindings) {
-		mlview_utils_trace_info ("System may be out of memory") ;
+		mlview_utils_trace_debug ("System may be out of memory") ;
 		return MLVIEW_OUT_OF_MEMORY_ERROR ;
 	}
 	memset (new_keybindings, 0, size) ;	
@@ -284,12 +284,12 @@ mlview_kb_eng_new (void)
 
 	result = g_try_malloc (sizeof (MlViewKBEng)) ;
 	if (!result) {
-		mlview_utils_trace_info ("System may be out of memory") ;
+		mlview_utils_trace_debug ("System may be out of memory") ;
 	}
 	memset (result, 0, sizeof (MlViewKBEng)) ;
 	PRIVATE (result) = g_try_malloc (sizeof (MlViewKBEngPriv)) ;
 	if (!PRIVATE (result)) {
-		mlview_utils_trace_info ("System may be out of memory") ;
+		mlview_utils_trace_debug ("System may be out of memory") ;
 	}
 	memset (PRIVATE (result), 0, sizeof (MlViewKBEngPriv)) ;
 	
@@ -373,21 +373,25 @@ mlview_kb_eng_lookup_a_key_binding (MlViewKBEng *a_this,
                         if (j >= a_key_input_tab_len) {
                                 key_sequence_too_short = TRUE ;
                                 global_key_sequence_too_short = TRUE ;
-#ifdef MLVIEW_VERBOSE
-                                g_print ("end of input key buf\n") ;
-                                g_print ("index in input key buf:%d \n", j) ;
-#endif                                
+                                mlview_utils_trace_debug 
+					("end of input key buf\n") ;
+                                mlview_utils_trace_debug 
+					("index in input key buf:%d \n", j) ;
                                 break ;
 
                         }
-#ifdef MLVIEW_VERBOSE
-                        g_print ("trying ro recon key: %x; mod & default mod mask:%x ...\n", 
-                                 a_key_input_tab[j].key, 
-                                 a_key_input_tab[j].modifier_mask & default_mod_mask) ;
-#endif
-                        g_print ("Current key in lookup sequence: %x, modifier & default_mode_mask: %x\n", 
-                                 PRIVATE (a_this)->keybindings[i].key_inputs[j].key,
-                                 PRIVATE (a_this)->keybindings[i].key_inputs[j].modifier_mask & default_mod_mask) ;
+
+                        mlview_utils_trace_debug 
+			("trying ro recon key: %x; "
+			 "mod & default mod mask:%x ...\n", 
+			 a_key_input_tab[j].key, 
+			 a_key_input_tab[j].modifier_mask & default_mod_mask) ;
+
+                        mlview_utils_trace_debug 
+			("Current key in lookup sequence: %x, "
+			 "modifier & default_mode_mask: %x\n", 
+			 PRIVATE (a_this)->keybindings[i].key_inputs[j].key,
+			 PRIVATE (a_this)->keybindings[i].key_inputs[j].modifier_mask & default_mod_mask) ;
 
                         if (PRIVATE (a_this)->keybindings[i].key_inputs[j].key 
                             != a_key_input_tab[j].key
@@ -400,34 +404,23 @@ mlview_kb_eng_lookup_a_key_binding (MlViewKBEng *a_this,
                 if (get_out == FALSE
                     && key_sequence_too_short == FALSE) {
                         found = TRUE ;
-#ifdef MLVIEW_VERBOSE
-                        g_print ("recognized key sequence (ok, woohoo)\n") ;
-#endif                                
+                        mlview_utils_trace_debug ("recognized key sequence (ok, woohoo)\n") ;
                         break ;                        
-                }
-#ifdef MLVIEW_VERBOSE
-                else {
-                        g_print ("didn't recognized key (failed)\n") ;
+                } else {
+                        mlview_utils_trace_debug ("didn't recognized key (failed)\n") ;
                 }                
-#endif
                 key_sequence_too_short = FALSE ;
         }
         if (found == TRUE) {
                 *a_key_binding_found = &PRIVATE (a_this)->keybindings[i] ;
-#ifdef MLVIEW_VERBOSE
-                g_print ("found keybinding %s\n",
+                mlview_utils_trace_debug ("found keybinding %s\n",
                          PRIVATE (a_this)->keybindings[i].name) ;
-#endif                                                
         } else if (global_key_sequence_too_short == TRUE){
                 status = MLVIEW_KEY_SEQUENCE_TOO_SHORT_ERROR ;
-#ifdef MLVIEW_VERBOSE
-                g_print ("input sequence too short\n") ;
-#endif
+                mlview_utils_trace_debug ("input sequence too short\n") ;
         } else {
                 status = MLVIEW_KEY_BINDING_NOT_FOUND_ERROR ;
-#ifdef MLVIEW_VERBOSE
-                g_print ("didn't find keybinding\n") ;
-#endif                                                                
+                mlview_utils_trace_debug ("didn't find keybinding\n") ;
         }
         return status ;
 }
@@ -458,36 +451,32 @@ mlview_kb_lookup_key_binding_from_key_press (MlViewKBEng *a_this,
                                                      &keybinding) ;
         if (status == MLVIEW_OK) {
                 if (keybinding) {
-#ifdef MLVIEW_VERBOSE
-                        g_print ("found a keybinding\n") ;
-#endif
+                        mlview_utils_trace_debug ("found a keybinding\n") ;
                         if (keybinding->action) {
-#ifdef MLVIEW_VERBOSE
-                                g_print ("Found an action assicated to keybinding: %s\n",
+                                mlview_utils_trace_debug 
+					("Found an action assicated to "
+					 "keybinding: %s\n",
                                          keybinding->name) ;
-#endif
                                 *a_key_binding_found = keybinding ;
                                 mlview_kb_eng_clear_key_inputs_queue (a_this) ;
                         } else {
-#ifdef MLVIEW_VERBOSe
-                                g_print ("No action was associated to the keybinding found\n") ;
-#endif
+                                mlview_utils_trace_debug 
+					("No action was associated "
+					 "to the keybinding found\n") ;
                         }
                 } else {
-                        g_print ("Found a NULL keybinding ... weird\n") ;
+                        mlview_utils_trace_debug 
+				("Found a NULL keybinding ... weird\n") ;
                         status = MLVIEW_ERROR ;                        
                 }
         } else if (status != MLVIEW_KEY_SEQUENCE_TOO_SHORT_ERROR) {
-#ifdef MLVIEW_VERBOSE
-                g_print ("No associated keybinding were found\n") ;
+                mlview_utils_trace_debug 
+			("No associated keybinding were found\n") ;
                 mlview_kb_eng_clear_key_inputs_queue (a_this) ;
-#endif                
+        } else {
+                mlview_utils_trace_debug 
+			("Only Found the begining of a keybinding\n") ;
         }
-#ifdef MLVIEW_VERBOSE
-        else {
-                g_print ("Only Found the begining of a keybinding\n") ;
-        }
-#endif
         return status ;
 }
 
