@@ -309,10 +309,6 @@ mlview_node_type_picker_init (MlViewNodeTypePicker * a_this)
 }
 
 
-/**
- *
- *
- */
 static void
 mlview_node_type_picker_update_node_type_list_and_elements_list (MlViewNodeTypePicker * a_this) 
 {
@@ -425,8 +421,7 @@ mlview_node_type_picker_update_node_type_list_and_elements_list (MlViewNodeTypeP
  *
  */
 static void
-mlview_node_type_picker_init_node_type_list (MlViewNodeTypePicker
-                                             * a_this)
+mlview_node_type_picker_init_node_type_list (MlViewNodeTypePicker* a_this)
 {
         int i = 0;
 
@@ -776,7 +771,7 @@ mlview_node_type_picker_get_node_name_or_content (MlViewNodeTypePicker * a_this)
                  *invocation of 
                  *mvliew_node_type_picker_build_element_name_choice_list ()
                  */
-                return (guchar *) gtk_entry_get_text
+                return (gchar *) gtk_entry_get_text
                         (GTK_ENTRY
                          (PRIVATE (a_this)->
                           node_name_or_content->entry));
@@ -957,7 +952,7 @@ mlview_node_type_picker_set_selected_node_type (MlViewNodeTypePicker * a_this,
                         &gv_xml_node_types[EXTERNAL_PARAMETER_ENTITY_NODE] ;
                         break ;
                 default:
-                        mlview_utils_trace_info 
+                        mlview_utils_trace_debug 
                                 ("should not reach this case") ;
                         break ;
                 }
