@@ -44,6 +44,7 @@ struct _MlViewNSEditorPrivate {
 	xmlNode *cur_xml_node ;
 	MlViewXMLDocument *xml_doc ;
 	MlViewAppContext *app_context ;
+        gboolean enable_node_alteration ;
         gboolean dispose_has_run ;        
 };
 
@@ -220,7 +221,7 @@ mlview_ns_editor_init (MlViewNSEditor *a_this)
         PRIVATE (a_this) = g_try_malloc 
                 (sizeof (MlViewNSEditorPrivate)) ;
         if (!PRIVATE (a_this)) {
-                mlview_utils_trace_info ("g_try_malloc failed") ;
+                mlview_utils_trace_debug ("g_try_malloc failed") ;
                 return ;
         }
         memset (PRIVATE (a_this), 0, sizeof (MlViewNSEditorPrivate)) ;
@@ -287,6 +288,11 @@ mlview_ns_editor_construct (MlViewNSEditor *a_this,
                  "editable", IS_EDITABLE_COLUMN,
                  NULL) ;
         scr_win = gtk_scrolled_window_new (NULL, NULL) ;
+	gtk_scrolled_window_set_policy (
+			GTK_SCROLLED_WINDOW (scr_win),
+			GTK_POLICY_AUTOMATIC,
+			GTK_POLICY_AUTOMATIC);
+	
         gtk_container_add (GTK_CONTAINER (scr_win),
                            GTK_WIDGET (PRIVATE (a_this)->view)) ;
         gtk_box_pack_start (GTK_BOX (a_this), scr_win,
@@ -297,7 +303,11 @@ mlview_ns_editor_construct (MlViewNSEditor *a_this,
                           "changed",
                           G_CALLBACK (row_selected_cb),
                           a_this) ;
+
+        PRIVATE (a_this)->enable_node_alteration = TRUE ;
+
         return MLVIEW_OK ;
+        
 }
 
 static enum MlViewStatus
@@ -324,7 +334,7 @@ mlview_ns_editor_select_ns (MlViewNSEditor *a_this,
         g_return_val_if_fail (tree_path, MLVIEW_ERROR) ;
         model = mlview_ns_editor_get_model (a_this) ;
         if (!model) {
-                mlview_utils_trace_info
+                mlview_utils_trace_debug
                         ("mlview_ns_editor_get_model() failed") ;
                 status = MLVIEW_ERROR ;
                 goto cleanup ;
@@ -332,7 +342,7 @@ mlview_ns_editor_select_ns (MlViewNSEditor *a_this,
         sel = gtk_tree_view_get_selection 
                 (PRIVATE (a_this)->view) ;
         if (!sel) {
-                mlview_utils_trace_info 
+                mlview_utils_trace_debug 
                         ("gtk_tree_model_get_selection () failed") ;
                 status = MLVIEW_ERROR ;
                 goto cleanup ;
@@ -389,6 +399,9 @@ ns_uri_cell_edited_cb (GtkCellRendererText *a_renderer,
         g_return_if_fail (PRIVATE (editor) 
                           && PRIVATE (editor)->cur_xml_node) ;
 
+        if (PRIVATE (editor)->enable_node_alteration == FALSE)
+                return ;
+
         status = mlview_ns_editor_get_cur_sel_start (editor, &iter) ;
         g_return_if_fail (status == MLVIEW_OK) ;
         model = mlview_ns_editor_get_model (editor) ;
@@ -428,7 +441,7 @@ ns_uri_cell_edited_cb (GtkCellRendererText *a_renderer,
                                  PRIVATE (editor)->cur_xml_node,
                                  TRUE) ;
                 } else {
-                        mlview_utils_trace_info
+                        mlview_utils_trace_debug
                                 ("We shouldn't be in this case") ;
                 }
         }
@@ -455,6 +468,10 @@ ns_prefix_cell_edited_cb (GtkCellRendererText *a_renderer,
 
         status = mlview_ns_editor_get_cur_sel_start (editor, &iter) ;
         g_return_if_fail (status == MLVIEW_OK) ;
+
+        if (PRIVATE (editor)->enable_node_alteration == FALSE)
+                return ;
+
         model = mlview_ns_editor_get_model (editor) ;
         if (mlview_ns_editor_is_row_the_add_new_ns_row
             (editor, &iter) == TRUE) {                
@@ -700,7 +717,7 @@ mlview_ns_editor_get_row_ref_from_iter (MlViewNSEditor *a_this,
         g_return_val_if_fail (tree_path, NULL) ;
         row_ref = gtk_tree_row_reference_new (model, tree_path) ;
         if (!row_ref) {
-                mlview_utils_trace_info 
+                mlview_utils_trace_debug 
                         ("gtk_tree_row_reference_new () failed") ;
                 result = NULL ;
                 goto cleanup ;
@@ -709,7 +726,7 @@ mlview_ns_editor_get_row_ref_from_iter (MlViewNSEditor *a_this,
                 PRIVATE (a_this)->ns_row_hash = g_hash_table_new
                         (g_direct_hash, g_direct_equal) ;
                 if (!PRIVATE (a_this)->ns_row_hash) {
-                        mlview_utils_trace_info
+                        mlview_utils_trace_debug
                                 ("g_hash_table_new() failed") ;
                         result = NULL ;
                         goto cleanup ;
@@ -841,6 +858,8 @@ mlview_ns_editor_edit_namespace (MlViewNSEditor * a_this,
                               && PRIVATE (a_this),
                               MLVIEW_BAD_PARAM_ERROR) ;
 
+        mlview_ns_editor_enable_node_alteration (a_this, FALSE) ;
+
         model = mlview_ns_editor_get_model (a_this) ;
         g_return_val_if_fail (model, MLVIEW_ERROR) ;
         is_ok = gtk_tree_model_get_iter_first (model, &iter) ;
@@ -870,7 +889,10 @@ mlview_ns_editor_edit_namespace (MlViewNSEditor * a_this,
         row_ref = mlview_ns_editor_get_row_ref_from_iter
                 (a_this, &iter2, TRUE/*create if not exist*/) ;
         g_return_val_if_fail (row_ref, MLVIEW_ERROR) ;
-        mlview_ns_editor_select_ns (a_this, a_ns) ;        
+        mlview_ns_editor_select_ns (a_this, a_ns) ;
+
+        mlview_ns_editor_enable_node_alteration (a_this, TRUE) ;
+
         return MLVIEW_OK ;
 }
 
@@ -922,7 +944,10 @@ mlview_ns_editor_edit_node_visible_namespaces (MlViewNSEditor * a_this,
                               && a_xml_node,
                               MLVIEW_BAD_PARAM_ERROR) ;
 
-        mlview_ns_editor_clear (a_this) ;
+        mlview_ns_editor_enable_node_alteration (a_this,
+                                                 FALSE) ;
+
+        mlview_ns_editor_clear (a_this) ;        
         node = a_xml_node ;
         while (node) {
                 editable = (node == a_xml_node) ;
@@ -938,6 +963,8 @@ mlview_ns_editor_edit_node_visible_namespaces (MlViewNSEditor * a_this,
                 node = node->parent ;
         }
         PRIVATE (a_this)->cur_xml_node = a_xml_node ;
+        mlview_ns_editor_enable_node_alteration (a_this,
+                                                 TRUE) ;
         return MLVIEW_OK ;
 }
 
@@ -1089,7 +1116,7 @@ mlview_ns_editor_update_ns (MlViewNSEditor *a_this,
         g_return_val_if_fail (tree_path, MLVIEW_ERROR) ;        
         is_ok = gtk_tree_model_get_iter (model, &iter, tree_path) ;
         if (is_ok != TRUE) {
-                mlview_utils_trace_info 
+                mlview_utils_trace_debug 
                         ("gtk_tree_model_get_iter() failed") ;
                 status = MLVIEW_ERROR ;
                 goto cleanup ;
@@ -1098,7 +1125,7 @@ mlview_ns_editor_update_ns (MlViewNSEditor *a_this,
                             XML_NS_COLUMN, &ns,
                             -1) ;
         if (!ns) {
-                mlview_utils_trace_info
+                mlview_utils_trace_debug
                         ("argh!! this column should have an xmlNs* "
                          "associated") ;
                 status = MLVIEW_ERROR ;
@@ -1242,3 +1269,15 @@ mlview_ns_editor_disconnect_from_doc (MlViewNSEditor *a_this,
                  a_this) ;
         return MLVIEW_OK ;
 }
+
+enum MlViewStatus
+mlview_ns_editor_enable_node_alteration (MlViewNSEditor *a_this,
+                                         gboolean a_enable)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_NS_EDITOR (a_this)
+                              && PRIVATE (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        PRIVATE (a_this)->enable_node_alteration = a_enable ;
+        return MLVIEW_OK ;
+}
