@@ -88,10 +88,11 @@ static xmlParserInput *mlview_sax_resolve_entity (void *a_ctx,
                                                   const xmlChar * a_public_id,
                                                   const xmlChar * a_system_id);
 
-static enum MlViewStatus load_xml_document_from_local_file (gchar * a_xml_file_name,
-                                                            xmlParserCtxtPtr * a_parser_context,
-                                                            gboolean a_store_external_subset_info,
-                                                            MlViewAppContext * a_app_context);
+static enum MlViewStatus load_xml_document_from_local_file 
+					(const gchar * a_file_name,
+				         xmlParserCtxtPtr * a_parser_context,
+				         gboolean a_store_external_subset_info,
+				         MlViewAppContext * a_app_context);
 
 static xmlDtdPtr custom_xmlSAXParseDTD (MlViewAppContext * a_app_context,
                                         xmlSAXHandlerPtr sax,
@@ -211,7 +212,7 @@ mlview_ext_subs_def_destroy (MlViewExtSubsDef * a_def)
  *to report  MLVIEW_ERRORS 
  */
 static enum MlViewStatus
-load_xml_document_from_local_file (gchar * a_xml_file_uri,
+load_xml_document_from_local_file (const gchar * a_xml_file_uri,
                                    xmlParserCtxt ** a_parser_context,
                                    gboolean a_store_external_subset_info,
                                    MlViewAppContext * a_app_context) 
@@ -361,7 +362,7 @@ mlview_external_entity_loader (const char *URL,
                 g_return_val_if_fail (fd, NULL);
                 error = mlview_file_descriptor_is_local (fd,
                                                          &is_local);
-                if (error || is_local != TRUE) {
+                if (error) {
                         if (fd) {
                                 mlview_file_descriptor_destroy
                                         (fd);
@@ -718,7 +719,7 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
                 base_uri = mlview_utils_get_dir_name_from_uri (a_doc->URL);
 
                 if (!base_uri) {
-                        mlview_utils_trace_info ("Can't get base URI.");
+                        mlview_utils_trace_debug ("Can't get base URI.");
 
                         g_free (system_id);
 
@@ -732,7 +733,7 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
                         base_uri = NULL;
 
                         if (!tmp_str || status != MLVIEW_OK) {
-                                mlview_utils_trace_info ("Can't get absolute URI.");
+                                mlview_utils_trace_debug ("Can't get absolute URI.");
 
                                 g_free (system_id);
                                 
@@ -817,7 +818,7 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
                                 mlview_ext_subs_def_new 
                                 ("", public_id, system_id) ;
                         if (! (*a_ext_subs_def)) {
-                                mlview_utils_trace_info 
+                                mlview_utils_trace_debug 
                                         ("Out of memory") ;
                                 goto cleanup ;
                         }
@@ -973,6 +974,7 @@ build_required_element_content (MlViewAppContext * a_app_context,
         xmlElementContent *current_element_content = a_content;
         xmlNode *child_node = NULL,
                 *dummy = NULL;
+	xmlDict *dict = NULL ;
 
         g_return_if_fail (a_app_context != NULL);
         g_return_if_fail (MLVIEW_IS_APP_CONTEXT (a_app_context));
@@ -982,8 +984,11 @@ build_required_element_content (MlViewAppContext * a_app_context,
         if (!current_element_content)
                 return;
 
+	if ((*a_node)->doc && (*a_node)->doc->dict) {
+		dict = (*a_node)->doc->dict ; 
+	}
         dummy = xmlNewNode (NULL, "<!dummy>");
-        xmlFree ((xmlChar *) dummy->name);
+        xmlDictFreeMem (dict, (xmlChar *) dummy->name);
         dummy->name = NULL;
 
         switch (current_element_content->type) {
@@ -1082,7 +1087,7 @@ build_required_element_content (MlViewAppContext * a_app_context,
                         }
                         break;
                 default:
-                        break;
+                        /*break*/;
                 }
                 break;
 
@@ -1154,8 +1159,8 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  *the parsing or NULL if the parsing failed. 
  */
 xmlDocPtr
-mlview_parsing_utils_load_xml_file_with_dtd (gchar *a_file_name,
-                                             gchar *a_dtd_name,
+mlview_parsing_utils_load_xml_file_with_dtd (const gchar *a_file_name,
+                                             const gchar *a_dtd_name,
                                              MlViewAppContext *a_app_context)
 {
         xmlDoc *result = NULL;
@@ -1193,16 +1198,18 @@ mlview_parsing_utils_load_xml_file_with_dtd (gchar *a_file_name,
                 }
         }
         
-        if (result && a_dtd_name) {
-                dtd = mlview_parsing_utils_load_dtd (a_dtd_name,
-                                                     a_app_context);
-                
-                if (dtd) {
-                        result->standalone = TRUE;
-                        result->extSubset = dtd;
-                        
-                        dtd->doc = result;
-                }
+        if (result) {
+		if (a_dtd_name && strcmp (a_dtd_name, "")) {
+			dtd = mlview_parsing_utils_load_dtd (a_dtd_name,
+							     a_app_context);
+			
+			if (dtd) {
+				result->standalone = TRUE;
+				result->extSubset = dtd;
+				
+				dtd->doc = result;
+			}
+		}
         }
         
         if (parser_context != NULL) {
@@ -1224,8 +1231,9 @@ mlview_parsing_utils_load_xml_file_with_dtd (gchar *a_file_name,
  *the parsing or NULL if the parsing failed. 
  */
 xmlDocPtr 
-mlview_parsing_utils_load_xml_file_with_dtd_interactive (gchar *a_file_uri,
-                                                         MlViewAppContext *a_app_context)
+mlview_parsing_utils_load_xml_file_with_dtd_interactive 
+					(const gchar *a_file_uri,
+                                         MlViewAppContext *a_app_context)
 {
         xmlDoc *result = NULL;
         xmlParserCtxtPtr parser_context = NULL;
@@ -1267,22 +1275,6 @@ mlview_parsing_utils_load_xml_file_with_dtd_interactive (gchar *a_file_uri,
         return result;
 }
 
-/**
- *Saves an xml file using the application wide settings. 
- *@param a_xml_doc the xml document to save.
- *@param a_app_context the current xml app context.
- *@return the number of bytes written or -1 in case of an error. 
- */
-gint
-mlview_parsing_utils_save_xml_doc (xmlDoc * a_xml_doc,
-                                   gchar * a_file_path,
-                                   MlViewAppContext *a_app_context)
-{
-        g_return_val_if_fail (a_xml_doc != NULL, -1);
-        g_return_val_if_fail (a_file_path != NULL, -1);
-
-        return xmlSaveFormatFile (a_file_path, a_xml_doc, 1);
-}
 
 /**
  *Loads an external DTD subset entity.
@@ -1502,7 +1494,7 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
 #else
 	gint response = 0;
 	GtkWidget *file_dialog = GTK_WIDGET (mlview_app_context_get_file_chooser 
-                                             (a_app_context, a_title)) ;
+                                             (a_app_context, a_title, MlViewFileChooserOpenMode)) ;
 
         g_return_val_if_fail (file_dialog != NULL, NULL);
 
@@ -2533,7 +2525,7 @@ mlview_parsing_utils_serialize_node_to_buf (const xmlNode *a_node,
 
         xml_buffer = xmlBufferCreate () ;
         if (!xmlNodeDump (xml_buffer, a_node->doc, (xmlNode*)a_node, 0, 0)){
-                mlview_utils_trace_info ("No byte were writen during serialization") ;
+                mlview_utils_trace_debug ("No byte were writen during serialization") ;
                 status = MLVIEW_ERROR ;
                 goto cleanup ;
         }
@@ -2614,7 +2606,7 @@ mlview_parsing_utils_parse_fragment (xmlDoc *a_doc,
         if (!doc)
                 doc = xmlNewDoc ("1.0") ;
         if (!doc) {
-                mlview_utils_trace_info ("Got a NULL document") ;
+                mlview_utils_trace_debug ("Got a NULL document") ;
                 return MLVIEW_ERROR ;
         }
         if (xmlParseBalancedChunkMemory (doc, NULL, NULL, 0, 
@@ -2636,3 +2628,53 @@ mlview_parsing_utils_parse_fragment (xmlDoc *a_doc,
 
         return status ;
 }
+
+enum MlViewStatus
+mlview_parsing_utils_uncomment_node (xmlDoc *a_doc,
+		                     const xmlNode *comment_node,
+		                     xmlNode **a_result_node)
+{
+	enum MlViewStatus status = MLVIEW_OK ;
+	xmlParserCtxt *parser_context = NULL ;
+	xmlNode *uncommented_node = NULL ;
+	gchar *content = NULL, *unescaped_content = NULL ;
+
+	g_return_val_if_fail (a_doc && comment_node && a_result_node,
+			      MLVIEW_BAD_PARAM_ERROR) ;
+	g_return_val_if_fail (comment_node->type == XML_COMMENT_NODE,
+			      MLVIEW_BAD_PARAM_ERROR) ;
+
+	content = xmlNodeGetContent ((xmlNode*)comment_node) ;
+	g_return_val_if_fail (content, MLVIEW_ERROR) ;
+
+	parser_context = xmlCreateMemoryParserCtxt (content, strlen (content)) ;
+	g_return_val_if_fail (parser_context, MLVIEW_ERROR) ;
+
+	unescaped_content = xmlStringDecodeEntities (parser_context,
+			                             content, 
+						     XML_SUBSTITUTE_REF,
+						     0, 0, 0) ;
+
+	g_return_val_if_fail (unescaped_content, MLVIEW_ERROR) ;
+
+	status = mlview_parsing_utils_parse_fragment (a_doc, unescaped_content,
+			                              &uncommented_node) ;
+	if (status != MLVIEW_OK) {
+		uncommented_node = xmlNewText (content) ;
+	}
+	if (content) {
+		g_free (content) ;
+		content = NULL ;
+	}
+	if (parser_context) {
+		xmlFreeParserCtxt (parser_context) ;
+		parser_context = NULL ;
+	}
+	if (unescaped_content) {
+		g_free (unescaped_content) ;
+		unescaped_content = NULL ;
+	}
+
+	*a_result_node = uncommented_node ;
+	return MLVIEW_OK ;
+}
