@@ -24,20 +24,45 @@
  */
 
 #include <glade/glade.h>
+#include <gnome.h>
 
 #include "mlview-schemas-window.h"
 #include "mlview-file-selection.h"
 #include "mlview-schema-list.h"
 
+static const gchar 
+* schemas_type_labels [] = {
+	"Unknown Schema",
+	"Document Type Definition",
+	"Relax-NG Schema",
+	"XML Schema Definition"
+};
+
+static const gchar*
+schemas_type_iconpath [] = {
+	NULL,
+	"mlview/mlview-dtd-schema.png",
+	"mlview/mlview-rng-schema.png",
+	"mlview/mlview-xsd-schema.png"
+};
+
 typedef struct _MlViewSchemasWindow MlViewSchemasWindow;
+typedef struct _MlViewSchemasWindowAddDialog MlViewSchemasWindowAddDialog;
+
+struct _MlViewSchemasWindowAddDialog {
+	GtkDialog *dialog;
+	GnomeFileEntry *fileentry;
+	GtkComboBox *combo;
+};
 
 struct _MlViewSchemasWindow {
-        GtkComboBox *combo;
         GtkTreeView *view;
         GtkWindow *win;
         MlViewAppContext *ctx;
         MlViewSchemaList *schemas;
         GHashTable *map;
+	
+	MlViewSchemasWindowAddDialog *add_dialog;
 };
 
 struct BuildModelData {
@@ -50,61 +75,67 @@ add_schema_to_list_store (MlViewSchema *a_schema,
                           GtkListStore *a_store,
                           GHashTable *a_table)
 {
-        enum MlViewSchemaType schema_type = SCHEMA_TYPE_UNDEF ;
-        enum MlViewStatus status = MLVIEW_OK ;
-        GtkTreeIter iter = { 0 };
-        gchar *url = NULL;
-        GtkTreePath *path = NULL;
-        GtkTreeRowReference *ref = NULL;
+	enum MlViewSchemaType schema_type = SCHEMA_TYPE_UNDEF ;
+	enum MlViewStatus status = MLVIEW_OK ;
+	GtkTreeIter iter = { 0 };
+	GtkTreePath *path = NULL;
+	GtkTreeRowReference *ref = NULL;
+	GdkPixbuf *pixbuf = NULL;
+	gchar *label = NULL,
+	      *url = NULL,
+	      *icon_path = NULL;
+	
+	g_return_if_fail (a_schema);
+	g_return_if_fail (a_store && GTK_IS_LIST_STORE (a_store));
+	g_return_if_fail (a_table);
+	
+	gtk_list_store_append (a_store, &iter);
+	
+	status = mlview_schema_get_type (a_schema, &schema_type) ;
+	g_return_if_fail (
+			status == MLVIEW_OK && schema_type != SCHEMA_TYPE_UNDEF) ;
+	
+	url = mlview_schema_get_url (a_schema);
+	label = g_strdup_printf ("%s\n<span color=\"gray\">%s</span>",
+			url, schemas_type_labels[schema_type]);
+	
+	icon_path = gnome_program_locate_file (
+			NULL,
+			GNOME_FILE_DOMAIN_APP_DATADIR,
+			schemas_type_iconpath[schema_type],
+			TRUE,
+			NULL);
 
-        g_return_if_fail (a_schema);
-        g_return_if_fail (a_store && GTK_IS_LIST_STORE (a_store));
-        g_return_if_fail (a_table);
-
-        gtk_list_store_append (a_store, &iter);
-
-        url = mlview_schema_get_url (a_schema);
-
-        gtk_list_store_set (a_store, &iter, 0, a_schema,
-                            2, url, -1);
-
-        status = mlview_schema_get_type (a_schema, &schema_type) ;
-        g_return_if_fail (status == MLVIEW_OK 
-                          && schema_type != SCHEMA_TYPE_UNDEF) ;
-
-        switch (schema_type) {
-        case SCHEMA_TYPE_DTD:
-                gtk_list_store_set (a_store, &iter, 1, "DTD", -1);
-                break;
-        case SCHEMA_TYPE_RNG:
-                gtk_list_store_set (a_store, &iter, 1, "Relax-NG", -1);
-                break;
-        case SCHEMA_TYPE_XSD:
-                gtk_list_store_set (a_store, &iter, 1, "XSD", -1);
-                break;
-        default:
-                g_assert_not_reached () ;
-        }
-
-        path = gtk_tree_model_get_path (GTK_TREE_MODEL (a_store),
-                                        &iter);
-
-        if (!path) {
-                gtk_list_store_remove (a_store, &iter);
-
-                return;
-        }
-
-        ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (a_store),
-                                          path);
-
-        if (ref) 
-                g_hash_table_insert (a_table, a_schema, ref);
-        else
-                gtk_list_store_remove (a_store, &iter); 
-        
-        gtk_tree_path_free (path);
-        path = NULL;
+	pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
+	
+	g_free (icon_path);
+	icon_path = NULL;
+	
+	gtk_list_store_set (a_store, &iter, 
+			0, a_schema,
+			1, GDK_PIXBUF (pixbuf),
+			2, label, 
+			-1);
+	
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (a_store),
+			&iter);
+	
+	if (!path) {
+		gtk_list_store_remove (a_store, &iter);
+		return;
+	}
+	
+	ref = gtk_tree_row_reference_new (
+			GTK_TREE_MODEL (a_store),
+			path);
+	
+	if (ref)
+		g_hash_table_insert (a_table, a_schema, ref);
+	else
+		gtk_list_store_remove (a_store, &iter); 
+	
+	gtk_tree_path_free (path);
+	path = NULL;
 }
 
 static void
@@ -120,7 +151,7 @@ add_schema_to_list_store_foreach (MlViewSchema *a_schema,
         add_schema_to_list_store (a_schema, a_data->store,
                                   a_data->table);
 }
-                          
+
 static void
 schemas_window_build_model_with_schemas (MlViewSchemaList *a_schemas,
                                          GtkTreeModel **a_model, 
@@ -132,13 +163,13 @@ schemas_window_build_model_with_schemas (MlViewSchemaList *a_schemas,
 
         g_return_if_fail (a_model && a_table);
         g_return_if_fail (a_schemas && MLVIEW_IS_SCHEMA_LIST (a_schemas));
-        
+
         store = gtk_list_store_new (3, G_TYPE_POINTER,
-                                    G_TYPE_STRING, G_TYPE_STRING);
+                                    GDK_TYPE_PIXBUF, G_TYPE_STRING);
 
         if (!store) 
                 goto cleanup;
-        
+
         table = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                                        NULL, (GDestroyNotify) gtk_tree_row_reference_free);
 
@@ -147,19 +178,19 @@ schemas_window_build_model_with_schemas (MlViewSchemaList *a_schemas,
 
         data = g_try_malloc (sizeof (struct BuildModelData));
 
-        if (!data) 
+        if (!data)
                 goto cleanup;
 
         data->store = store;
         data->table = table;
-        
+
         mlview_schema_list_foreach (a_schemas, 
                                     (MlViewSchemaListFunc) add_schema_to_list_store_foreach,
                                     data);
 
         g_free (data);
         data = NULL;
-       
+
         *a_model = GTK_TREE_MODEL (store);
         *a_table = table;
 
@@ -188,36 +219,61 @@ schemas_window_build_model_with_schemas (MlViewSchemaList *a_schemas,
 static void 
 schemas_window_add_clicked_cb (GtkButton *a_button, MlViewSchemasWindow *a_schemas)
 {
-        gint index = -1;
-        MlViewSchema *schema = NULL;
-
-        g_return_if_fail (a_schemas);
-        g_return_if_fail (a_schemas->combo && GTK_IS_COMBO_BOX (a_schemas->combo));
-        g_return_if_fail (a_schemas->schemas);
-        g_return_if_fail (a_schemas->ctx && MLVIEW_IS_APP_CONTEXT (a_schemas->ctx));
-        
-        index = gtk_combo_box_get_active (a_schemas->combo);
-
-        if (index == -1)
-                return;
-
-        switch (index) {
-        case 0:
-                schema = mlview_schema_load_interactive
-                        (SCHEMA_TYPE_DTD, a_schemas->ctx);
-                break;
-        case 1:
-                schema = mlview_schema_load_interactive
-                        (SCHEMA_TYPE_RNG, a_schemas->ctx);
-                break;
-        case 2:
-                schema = mlview_schema_load_interactive
-                        (SCHEMA_TYPE_XSD, a_schemas->ctx);
-                break;
-        }
-
+	gint result = -1;
+	gint index = -1;
+	gchar *schema_uri = NULL;
+	MlViewSchema *schema = NULL;
+	enum MlViewSchemaType schema_type = -1;
+	
+	g_return_if_fail (a_schemas);
+	g_return_if_fail (a_schemas->add_dialog);
+	g_return_if_fail (a_schemas->add_dialog->dialog);
+	g_return_if_fail (a_schemas->add_dialog->combo);
+	g_return_if_fail (a_schemas->add_dialog->fileentry);
+	g_return_if_fail (a_schemas->schemas);
+	g_return_if_fail (a_schemas->ctx && MLVIEW_IS_APP_CONTEXT (a_schemas->ctx));
+		
+	result = gtk_dialog_run (GTK_DIALOG (a_schemas->add_dialog->dialog));
+	switch (result) {
+	case GTK_RESPONSE_ACCEPT:
+		index = gtk_combo_box_get_active (a_schemas->add_dialog->combo);
+		schema_uri = (gchar*) gtk_entry_get_text (
+				(GtkEntry*)gnome_file_entry_gtk_entry (
+						GNOME_FILE_ENTRY (
+							a_schemas->add_dialog->fileentry)));
+		break;
+	default:
+		break;
+	}
+	gtk_widget_hide (GTK_WIDGET (a_schemas->add_dialog->dialog));
+	
+	if (index == -1)
+		return;
+	
+	switch (index) {
+	case 0:
+		schema_type = SCHEMA_TYPE_DTD;
+		break;
+	case 1:
+		schema_type = SCHEMA_TYPE_RNG;
+		break;
+	case 2:
+		schema_type = SCHEMA_TYPE_XSD;
+		break;
+	}
+	
+	schema = mlview_schema_load_from_file (
+			schema_uri,
+			schema_type,
+			a_schemas->ctx);
         if (schema)
-                mlview_schema_list_associate_schema (a_schemas->schemas, schema);
+                mlview_schema_list_add_schema (a_schemas->schemas, schema);
+
+	/* add schema to combo history */
+	gnome_entry_prepend_history (
+			GNOME_ENTRY (gnome_file_entry_gnome_entry (a_schemas->add_dialog->fileentry)),
+			TRUE,
+			schema_uri);
 }
 
 static void
@@ -257,7 +313,7 @@ unassociate_schema_foreach (GtkTreePath *a_path,
         if (!url)
                 goto cleanup;
 
-        res = mlview_schema_list_unassociate_schema_by_url 
+        res = mlview_schema_list_remove_schema_by_url 
                 (a_schemas->schemas, url);
         
         if (!res)
@@ -393,23 +449,28 @@ schemas_window_destroy_cb (GtkWidget *a_win,
         a_schemas = NULL;
 }
 
+
 GtkWidget *
 mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
                                          MlViewAppContext *a_ctxt)
 {
         GladeXML *gxml = NULL;
-        gchar *gfile = NULL, *tmp = NULL,
-                *url = NULL;
-        GtkWidget *win = NULL, *view = NULL,
-                *combo = NULL;
+        gchar *gfile = NULL,
+	      *tmp = NULL,
+	      *url = NULL;
+        GtkWidget *win = NULL,
+		  *view = NULL,
+		  *dialog = NULL,
+		  *combo = NULL,
+		  *fileentry = NULL;
         MlViewSchemaList *schemas = NULL;
         GtkTreeModel *model = NULL;
         GtkListStore *text = NULL;
         MlViewSchemasWindow *data = NULL;
         GHashTable *table = NULL;
         GtkTreeIter iter = { 0 };
-        GtkCellRenderer *renderer = NULL;
         GtkTreeSelection *selection = NULL;
+	GtkCellRenderer *renderer = NULL;
 
         g_return_val_if_fail (a_ctxt && MLVIEW_IS_APP_CONTEXT (a_ctxt), NULL);
         g_return_val_if_fail (a_doc && MLVIEW_IS_XML_DOCUMENT (a_doc), NULL);
@@ -436,7 +497,7 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
         if (!gxml)
                 goto cleanup;
         
-        win = glade_xml_get_widget (gxml, "SchemasWindow");
+        win = glade_xml_get_widget (gxml, "schemas_window");
         
         if (!(win && GTK_IS_WINDOW (win)))
                 goto cleanup;
@@ -444,7 +505,7 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
         url = mlview_xml_document_get_file_path (a_doc);
         
         if (url) {
-                tmp = g_strconcat ("Schemas - ", url, NULL);
+                tmp = g_strconcat ("Schemas", " - ", url, NULL);
                 
                 if (tmp) {
                         gtk_window_set_title (GTK_WINDOW (win), tmp);
@@ -460,7 +521,7 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
         if (!(model && GTK_IS_TREE_MODEL (model) && table))
                 goto cleanup;
         
-        view = glade_xml_get_widget (gxml, "SchemasView");
+        view = glade_xml_get_widget (gxml, "schemas_treeview");
 
         if (!(view && GTK_IS_TREE_VIEW (view)))
                 goto cleanup;
@@ -472,22 +533,32 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
 
         gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
-        renderer = gtk_cell_renderer_text_new ();
+        renderer = gtk_cell_renderer_pixbuf_new ();
         gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view), 
                                                      1, _("Type"),
-                                                     renderer, "text", 1, NULL);
+                                                     renderer, "pixbuf", 1, NULL);
 
         renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view), 
                                                      2, _("URL"),
-                                                     renderer, "text", 2, NULL);
+                                                     renderer, "markup", 2, NULL);
 
         gtk_tree_view_set_model (GTK_TREE_VIEW (view), model);
         
         g_object_unref (G_OBJECT (model));
         model = NULL;
 
-        combo = glade_xml_get_widget (gxml, "TypeSelector");
+	dialog = glade_xml_get_widget (gxml, "add_dialog");
+
+	if (!(dialog && GTK_IS_DIALOG (dialog)))
+		goto cleanup;
+
+	fileentry = glade_xml_get_widget (gxml, "fileentry");
+
+	if (!fileentry)
+		goto cleanup;
+
+	combo = glade_xml_get_widget (gxml, "combo");
         
         if (!(combo && GTK_IS_COMBO_BOX (combo)))
                 goto cleanup;
@@ -498,11 +569,11 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
                 goto cleanup;
         
         gtk_list_store_append (text, &iter);
-        gtk_list_store_set (text, &iter, 0, "DTD", -1);
+        gtk_list_store_set (text, &iter, 0, "Document Type Definition (DTD)", -1);
         gtk_list_store_append (text, &iter);
-        gtk_list_store_set (text, &iter, 0, "Relax-NG", -1);
+        gtk_list_store_set (text, &iter, 0, "Relax-NG Schema (RNG)", -1);
         gtk_list_store_append (text, &iter);
-        gtk_list_store_set (text, &iter, 0, "XSD", -1);
+        gtk_list_store_set (text, &iter, 0, "XML Schema Definition (XSD)", -1);
         
         renderer = gtk_cell_renderer_text_new ();
         
@@ -519,18 +590,23 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
      
         g_object_unref (G_OBJECT (text));
         text = NULL;
-        
+
+	
         data = g_try_malloc (sizeof (MlViewSchemasWindow));
         
         if (!data)
                 goto cleanup;
 
-        data->combo = GTK_COMBO_BOX (combo);
+	data->add_dialog = g_try_malloc (sizeof (MlViewSchemasWindowAddDialog));
+
         data->win = GTK_WINDOW (win);
         data->view = GTK_TREE_VIEW (view);
         data->ctx = a_ctxt;
         data->schemas = schemas;
         data->map = table;
+	data->add_dialog->dialog = GTK_DIALOG (dialog);
+        data->add_dialog->combo = GTK_COMBO_BOX (combo);
+	data->add_dialog->fileentry = GNOME_FILE_ENTRY (fileentry);
 
         glade_xml_signal_connect_data (gxml, "on_add_clicked", 
                                        G_CALLBACK (schemas_window_add_clicked_cb), 
@@ -573,6 +649,11 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
                 g_free (url);
                 url = NULL;
         }
+
+	if (dialog) {
+		gtk_widget_destroy (dialog);
+		dialog = NULL;
+	}
         
         if (tmp) {
                 g_free (tmp);
@@ -588,7 +669,12 @@ mlview_schemas_window_new_with_document (MlViewXMLDocument *a_doc,
                 g_object_unref (G_OBJECT (text));
                 text = NULL;
         }
-        
+
+	if (data->add_dialog) {
+		g_free (data->add_dialog);
+		data->add_dialog = NULL;
+	}
+
         if (data) {
                 g_free (data);
                 data = NULL;
